use std::collections::HashMap;

use ropey::Rope;
use sv_parser::{RefNode, SyntaxTree};
use serde_json::{Value, json};

use crate::core::sv_parser::{get_identifier, get_position};

struct CommentSymbol {
    comment_string: String,
    start_pos: crate::core::hdlparam::Position,
    end_pos: crate::core::hdlparam::Position
}

pub fn get_comments_from_ast(doc: &Rope, syntax_tree: &SyntaxTree) {
    
    let comments = make_comment_list(doc, syntax_tree);
    
    for comment in &comments {
        if let Some(comment_meta) = get_meta_info(comment) {

        } else {
            // 处理为普通注释
        }
    }
}

/// 创造一个收容 comment 的栈，对连续的注释进行合并
fn make_comment_list(doc: &Rope, syntax_tree: &SyntaxTree) -> Vec<CommentSymbol> {
    let mut comments = Vec::<CommentSymbol>::new();

    for node in syntax_tree {
        match node {
            RefNode::Comment(x) => {
                let locate = x.nodes.0;                
                let comment_string = syntax_tree.get_str(&locate).unwrap();
                let start_pos = get_position(doc, locate, 0);
                let end_pos = get_position(doc, locate, locate.len);

                println!("{:?}", comment_string);
                println!("start {:?}", start_pos);
                println!("end {:?}", end_pos);

                if let Some(last_comment) = comments.last_mut() {
                    // 判断是否需要合并
                    // vlog 只合并单行注释
                    let last_end_line = last_comment.end_pos.line;
                    let current_start_line = start_pos.line;

                    if current_start_line - last_end_line <= 1
                        && last_comment.comment_string.starts_with("//")
                        && comment_string.starts_with("//") {
                            last_comment.comment_string += comment_string;
                            last_comment.end_pos = end_pos;
                            continue;
                    }
                }

                // 否则，直接加入
                comments.push(CommentSymbol {
                    comment_string: comment_string.to_string(),
                    start_pos, end_pos
                });
            }

            _ => {}
        }
    }
    comments
}

struct CommentMeta<'a> {
    comment_type: &'a str,
    argruments: Vec::<&'a str>
}

/// 解析第一行 tag
/// /* @meta
/// * 
/// */
/// 
/// /* @module
/// * 
/// */
/// 
/// /* @wavedrom xxx
/// * 
/// */
/// 
/// 或者压根儿没有
/// /*
/// * 
/// */
fn get_meta_info(comment: &str) -> Option<CommentMeta> {
    let first_line = comment.lines().next().unwrap_or("").trim();
    // 注释开头都是 // 或者 /* ，都是两个字符
    let comment_meta = &first_line[2..];
    if comment_meta.trim().starts_with("@") {
        let arguments: Vec<&str> = comment_meta.split_whitespace().collect();
        if arguments.len() > 0 {
            let comment_type = arguments[0];
            let arguments = &arguments[1..];
            Some(CommentMeta {
                comment_type,
                argruments: arguments.to_vec()
            })
        } else {
            None
        }
    } else {
        None
    }
}

/// 解析不同的注释块
fn codedoc_parse_pipeline(comment: &str) {

}


fn parse_c_style_comment(comment: &str) -> Result<Value, String> {
    let mut result = HashMap::new();
    let mut revision_vec = Vec::new();
    let mut in_revision = false;

    for line in comment.lines() {
        let line = line.trim();
        if line.is_empty() || line.starts_with("/*") || line.starts_with("*/") {
            continue;
        }

        if line.starts_with("*") {
            let line = line[1..].trim();
            if line.starts_with("@meta") {
                continue;
            }

            if let Some((key, value)) = line.split_once(':') {
                let key = key.trim();
                let value = value.trim();

                match key {
                    "Tool Versions" => {
                        let tools: Vec<&str> = value.split('&').map(|s| s.trim()).collect();
                        result.insert(key.to_string(), json!(tools));
                    }
                    "Revision" => {
                        in_revision = true;
                    }
                    _ => {
                        if in_revision {
                            if value.matches('&').count() == 2 {
                                let parts: Vec<&str> = value.split('&').map(|s| s.trim()).collect();
                                let revision = json!({
                                    "date": parts[0],
                                    "version": parts[1],
                                    "revision": parts[2]
                                });
                                revision_vec.push(revision);
                            } else {
                                in_revision = false;
                                result.insert(key.to_string(), json!(revision_vec));
                            }
                        } else {
                            result.insert(key.to_string(), json!(value));
                        }
                    }
                }
            }
        }
    }

    // Handle Copyright field
    if let Some(copyright) = result.get_mut("Copyright") {
        let copyright_str = copyright.as_str().unwrap_or("");
        *copyright = json!(format!("Copyright(c) {}", copyright_str));
    }

    Ok(json!(result))
}